/*-----------------------------------------------------------------------------
| Copyright (c) Jupyter Development Team.
| Distributed under the terms of the Modified BSD License.
|----------------------------------------------------------------------------*/

/* This file was auto-generated by ensurePackage() in @jupyterlab/buildutils */
import '@jupyterlab/application/style/index.js';
import '@jupyterlab/docmanager/style/index.js';
import '@jupyterlab/cells/style/index.js';
import '@jupyterlab/notebook/style/index.js';
