/*-----------------------------------------------------------------------------
| Copyright (c) Jupyter Development Team.
| Distributed under the terms of the Modified BSD License.
|----------------------------------------------------------------------------*/

/* This file was auto-generated by ensurePackage() in @jupyterlab/buildutils */
@import url('~@jupyterlab/application/style/index.css');
@import url('~@jupyterlab/docmanager/style/index.css');
@import url('~@jupyterlab/cells/style/index.css');
@import url('~@jupyterlab/notebook/style/index.css');
